;Programm    : p2004201.asm  (ZD-931 Upgrade)
;Datum       : 06.07.2020
;Name		 : Dipl.-Ing. W. Sontopski
;Controller	 : ATmega168
;Clock       : extern 10.24Mhz
;Board       : p2004201
;Kunde       : soni
;Programmfile: LLxxxx12.elf
;--------------------------------------
;--------------------------------------	
;;ZD-931 Upgrade
;
;diese Hard-, Software ersetzt das Steuer-, Anzeigemodul
;in der Ltstation ZD-931.
;der Anschluss fr den Ltkolben wurde um 2 weitere
;Anschlsse ergnzt. Es knnen jetzt preiswerte 
;Ltkolben (Hakko-kompatible, 3-4 EUR, Aliexpress) 
;angeschlossen werden. Kolben mit Thermoelement als
;auch PTC als Temperaturfhler sind mglich. Die
;Auflsung des AD-Wandlers zur Temperaturmessung 
;ist 16 Bit, wird aber ber eine lookup Tabelle mittels
;linearer Interpolation auf 1 Grad  Auflsung runtergerechnet.
;Die Genauigkeit der Temperatur hngt direkt von der
;Anzahl und den Messwerten in der lookup Tabelle ab.
;Nichtliearitten der Temperaturerfassung werden damit 
;weitestgehend eliminiert.
;kleiner Hinweis: An vielen Stellen im Netz wird behauptet,
;das Thermolement in den Hakko-Ltklben sei ein K-Typ.
;Nach meinen Messungen handelt es sich aber um ein J-Typ.
;Dies ist allerdings bei meiner Methode ber eine 
;lookup Tabelle irrelevant. 		
;------------------------------------------------
;------------------------------------------------

.include	"m168def.inc"
.include	"macros4.inc"

.equ	true		= 0xff
.equ	false		= 0

.equ	cr		= 0x0d
.equ	lf		= 0x0a


; output stream destinations
.equ	null_stream		= 0	; nowhere
.equ	serial_stream	= 1	; serial out
.equ	memory_stream	= 2	; RAM
.equ	lcd_stream		= 3	; LCD screen

.equ	ram = 1			; used for the ldX/ldY/ldZ/ldACC macros
.equ	rom = 2

.def	zero		=	r2		; always contains zero
.def	temp1		=	r16		; temp register
.def	temp2		=	r17		; temp register
.def	temp3		=	r18		; temp register
.def	loopcounter =	r19		; used for loop counting
.equ	Uoff1		= 0x17ae	;offset chn.1 (x6), j-typ thermo.	

.equ	t_down			= 5
.equ	t_up			= 4
	




.equ	PWmin		= 0x09	;PW zu klein, (kurzschluss)?
.equ	PWmax		= 0xf0	;PW zu gross, (offen; Ue zu klein)?

.equ	tstart		= 250	;starttemperatur
.equ	tmax		= 450
.equ	tmin		= 100	

;diverse a/d-setup
.equ	pre_cnt		= 10		;einschwingzhler
.equ	u_cnt		= 50		;anzahl der untermessungen


.equ	time1		= 30	;wiederholzeit (sec) fuer eine dU-messung(30)
.equ	first		= 0
.def	flag		= r26
.def	status  	= r12
.def	cnt_u   	= r13
.def	adl			= r8
.def	adh			= r9

;flag-bits 
.equ	flg_invert	= 0	;invert-flag
.equ	flg_co		= 1	;console output-flag 0->lcd, 1->ser
.equ	ad_m		= 2
.equ	ad_rdy		= 3
.equ	chn_ok		= 4
.equ	kint1		= 5	;1= komparator (N4) ist gekommen 1->0


.dseg

chn:		.byte	1		;<- muss an 1. stelle stehen (SRAM_START)
ad_ch:		.byte	6		;3 ad-werte je 16bit
cnt1:		.byte	2
;------------------------
stream_destination:	.byte	1	; where an output stream is sent
					; (0 => serial; 1 => memory, lcd)
;===============================================================
row:		.byte	1	; LCD row pos for text cell positioning
coll:		.byte	1	; collum as above for text cell positioning

Lpage:		.byte	1	; LCD page. Display is 8 pages (rows of pixels, 8 bits high, vertical)
Lcoll:		.byte	1	; actual LCD collum address. Which byte of pixels (0...127) along the row
Lwidth:		.byte	1	; width of big LCD chars. Nums and dot/comma are different

;------------------------
start_time0:.byte	2		;abfolge so lassen
start_time1:.byte	2
msec:		.byte	2
t_sec:		.byte	2
t_min:		.byte	1
t_hr:		.byte	1
;------------------------
bcd0:		.byte	1
bcd1:		.byte	1
bcd2:		.byte	1
bcds:		.byte	6		;bcd temperatursollwert
bcdi:		.byte	6		;bcd temperaturistwert

tist:		.byte	2		;temperatur istwert (bin)
tsoll:		.byte	2		;temperatur sollwert (bin)


rx_pt:		.byte	1
rx:			.byte	5		;4 zahlen (0-9) +1x cr
ds_end:		.byte	1		;<- mu an letzter stelle stehen


	.eseg
ee_tsoll:	.byte	2		;temperatur sollwert (bin)


	.cseg

	.org	0
	rjmp	p2004_

	.org	0x0004			;int1 N4_2
	rjmp	acomp_int

	.org	0x012			;atmega168 timer2 overflow int.
	rjmp	t2ms

	.org	0x01a
	rjmp	tim1_ov_int		;timer1 overflow int.


;--------------------------------------------------------
;********************************************************

p2004_:
	ldi		r16,low(RAMEND)
	out		SPL,r16			;init. stackpointer
	ldi		r16,high(RAMEND)
	out		SPH,r16			;init. stackpointer
	rcall	wdt_off			;watchdog aus
	clr		flag			;alle flags =0
	ldi		r16,0b00000100	;
	out		PORTB,r16 
	ldi		r16,0b00010111
	out		DDRB,r16		;set portb i/o-config.(0=in)
	ldi		r16,0b00000100
	out		PORTC,r16 
	ldi		r16,0b10111111
	out		DDRC,r16		;set portc i/o-config.(0=in)
	ldi		r16,0b01110000
	out		PORTD,r16
	ldi		r16,0b11000011
	out		DDRD,r16		;set portd i/o-config.(0=in)
	rcall	del200ms
;	rjmp	ser_tst		;****test only
	rcall	clr_var
	rcall	get_tsoll		;tsoll ee -> ram
;	rcall	ram_dump	;****test only
	lds		r17,tsoll+1
	lds		r16,tsoll
;	rcall	lr17r16_hex_crlf
	cpi		r17,0xff		;schon ein wert im ee-prom??
	brne	p2004_10
	ldi		r16,low(tstart)
	ldi		r17,high(tstart)
;	rcall	lr17r16_hex_crlf
	sts		tsoll,r16		
	sts		tsoll+1,r17
	rcall	save_tsoll		;sichert solltemperatur -> ee
p2004_10:
;	rjmp	p2004_10	;****test only
	cli
	clr		zero
	andi	flag,0b11111100	;clr flags:flg_invert,flg_co
	rcall	init_st7565
	rcall	del50ms
	rcall	SollIstC		;txt  Soll:, Ist: C -> lcd
	rcall	chk_lk			;testet welcher ltkolben angeschlossen ist
	brcc	p2004_20
	lcd_coll 0
	lcd_page 6 
	ldi		zh,high(txt50<<1) ;"LTKOLBEN?"
	ldi		zl,low(txt50<<1)
	rcall	stringf
	rcall	del1s
;	rcall	lr17r16_hex_crlf	;****test only
	rjmp	p2004_10
p2004_20:
	rcall	ini_timer1		;ini. timer1, alle 2ms int.
	sei						;ints aktiv
p2004_22:
	lds		r16,tsoll
	lds		r17,tsoll+1
	sbis	pind,t_up		;i-up tastenabfrage
	rcall	tup
	sbis	pind,t_down		;i-down tastenabfrage
	rcall	tdown
;	rcall	lr17r16_hex_crlf	;****test only
	ldi		zl,low(bcds)
	ldi		zh,high(bcds)
	rcall	Bin2ToBcd5
	rcall	soll2lcd		;sollwert -> lcd
p2004_30:
	lds		r16,chn			;ad-kanal nr.
	tst		r16
	brne	p2004_32
	rcall	adu1			;1x adw j-typ
	brcs	p2004_10		;springt wenn kein Ltkolben
	rcall	jtyp			;j-typ korrektur
	rjmp	p2004_34
p2004_32:
	rcall	adu1			;1x adw ptc
	brcs	p2004_10		;springt wenn kein Ltkolben
	rcall	ptc				;ptc korrektur
p2004_34:
	sts		tist,r16
	sts		tist+1,r17
	ldi		zl,low(bcdi)
	ldi		zh,high(bcdi)
	rcall	Bin2ToBcd5
	rcall	ist2lcd			;istwert -> lcd
;	rjmp	p2004_30	;****test only
p2004_40:
	lds		r20,tsoll
	lds		r21,tsoll+1
	lds		r16,tist
	lds		r17,tist+1
	rcall	cp16			;test tsoll <=> tist
;	jmp		p2004_30	;****test only
	brcs	p2004_50
	rcall	triac_off		;tsoll > tist
	jmp		p2004_22
p2004_50:
	rcall	triac_on		;tsoll < tist
	clc
	jmp		p2004_22
		

;***********************************************
;Prft welcher ltkolben eingesteckt ist.
;Ist in einem kanal kein ltkolben eingesteckt,
;dann geht der integrator (N5) -> -u und 
;der komparator (N4) -> VCC_3.3
;ints mssen aus sein
;am ende:cy=0, chn = kanalnr. 
;        cy=1, kein ltkolben eingesteckt     

chk_lk:
	rcall	triac_off
	cbi		portc,2		;ref an
ckl_10:
	ldi		r16,0		;ad-kanal 0
	rcall	out_chn
	rcall	del10ms
	sbis	pind,3		;test komparatorausgang
	rjmp	ckl_20
	ldi		r16,2		;ad-kanal 2
	rcall	out_chn
	rcall	del10ms
	sbic	pind,3		;test komparatorausgang
	rjmp	ckl_err		;-> kein ltkolben
ckl_20:
	sts		chn,r16		;speichert ad-kanal-nr.
	clc
	ret
ckl_err:
	ldi		r16,0xff
	sec
	ret


;-----------------------------------------------
;testet ob tsoll_max schon erreicht ist, wenn nicht,
;dann wird tsoll = tsoll+1
tup:
	push	r21
	push	r20
	ldi		r20,low(tmax)
	ldi		r21,high(tmax)
	rcall	cp16
	brcc	tup_20
	subi	r16,-1
	sbci	r17,-1
tup_10:
	sts		tsoll,r16
	sts		tsoll+1,r17
	rcall	save_tsoll		;tsoll -> ee
tup_20:
	pop		r20
	pop		r21
	ret

;testet ob tsoll_min schon erreicht ist, wenn nicht,
;dann wird tsoll = tsoll-1

tdown:
	push	r21
	push	r20
	ldi		r20,low(tmin)
	ldi		r21,high(tmin)
	rcall	cp16
	brcs	tup_20
	subi	r16,1
	sbci	r17,0
	rjmp	tup_10



;-----------------------------------------------
;berechnet den temperaturwert (T) fr einen bestimmten
;messwert (Mw).
;es wurden n messwerte ermittelt, die als sttzpunkte
;zur wertberechnung genutzt werden.
;es wird gerechnet
;                                          Mw - Mlt[n-1]
;   T = Tlt[n-1] + (Tlt[n] - Tlt[n-1]) x -----------------
;                                        Mlt[n] - Mlt[n-1]
;
;Mlt = Messwert lookup table
;Tlt = Temperatur lookup table 
;
;zu beginn: r17:r16 Messwert Mw
;am ende  : r17:r16 Temperatur (1 Grad Auflsung)

ptc:
	push	zh
	push	zl
	push	r23
	push	r22
	push	r21
	push	r20
	push	r19
	push	r18
	push	r11
	push	r10
	push	r9
	push	r8
	push	r7
	push	r6
	push	r5
	push	r4
	push	r3
	push	r2
	push	r1
	push	r0
	ldi		zh,high(PTClt<<1)
	ldi		zl,low(PTClt<<1)
ptc10:
	lpm		r20,z+
	lpm		r21,z+
	rcall	cp16
	brcs	ptc20
	adiw	z,2
	rjmp	ptc10
ptc20:
	movw	r2,r20		;r3:r2 = Mlt[n]
	lpm		r20,z+
	lpm		r21,z+
	movw	r4,r20		;r5:r4 = Tlt[n]
	sbiw	z,8
	lpm		r20,z+
	lpm		r21,z+
	movw	r6,r20		;r7:r6 = Mlt[n-1]
	lpm		r20,z+
	lpm		r21,z+
	movw	r8,r20		;r9:r8 = Tlt[n-1]
	sub		r2,r6
	sbc		r3,r7		;r3:r2 = Mlt[n] - Mlt[n-1]
	sub		r16,r6
	sbc		r17,r7
	movw	r6,r16		;r7:r6 = Mw - Mlt[n-1]
	sub		r4,r8
	sbc		r5,r9		;r5:r4 = Tlt[n] - Tlt[n-1]
	movw	r16,r4
	movw	r18,r6
	rcall	MUL16x16	;(Tlt[n] - Tlt[n-1]) x (Mw - Mlt[n-1])
	movw	r16,r20
	movw	r18,r22
	movw	r20,r2
	rcall	DIV3216
	add		r8,r0
	adc		r9,r1
	movw	r16,r8
	pop		r0
	pop		r1
	pop		r2
	pop		r3
	pop		r4
	pop		r5
	pop		r6
	pop		r7
	pop		r8
	pop		r9
	pop		r10
	pop		r11
	pop		r18
	pop		r19
	pop		r20
	pop		r21
	pop		r22
	pop		r23
	pop		zl
	pop		zh
	ret

jtyp:
	push	zh
	push	zl
	push	r23
	push	r22
	push	r21
	push	r20
	push	r19
	push	r18
	push	r11
	push	r10
	push	r9
	push	r8
	push	r7
	push	r6
	push	r5
	push	r4
	push	r3
	push	r2
	push	r1
	push	r0
	ldi		zh,high(JTyplt<<1)
	ldi		zl,low(JTyplt<<1)
	rjmp	ptc10

	

;ptc lookup table
;12 ermittelte wertepaare: messwert, temperatur C
;die Messwerte wurde ermittelt mit einem
;digital thermometer 6802II (K-Typ Sonde)
;
PTClt:
	.dw	0x3cde, 0
	.dw	0x406e, 22
	.dw	0x45bc, 56
	.dw	0x4e69, 110
	.dw	0x565f, 162
	.dw	0x5dae, 216
	.dw	0x6729, 272
	.dw	0x6f83, 321
	.dw	0x79e9,	380
	.dw	0x820d,	427
	.dw	0x8e54, 485
	.dw	0x99b2, 500

;J-Typ lookup table
;10 ermittelte wertepaare: messwert, temperatur C
;die Messwerte wurde ermittelt mit einem
;digital thermometer 6802II (K-Typ Sonde)

JTyplt:
	.dw	0x1a71,	0
	.dw	0x3bf0,	91
	.dw	0x4ddf,	137
	.dw	0x5fd9,	196
	.dw	0x71b4,	240
	.dw	0x839d,	288
	.dw	0x9651,	332
	.dw	0xa840,	382
	.dw	0xba1e,	430
	.dw	0xcc0f,	470	

;----------------------------------------------
triac_on:
	push	r16
	ldi		r16,0b00100010	;timer fast pwm mode
	sts		TCCR1A,r16
	pop		r16
	ret	
triac_off:
	push	r16
	ldi		r16,0b00000000	;timer normal mode
	sts		TCCR1A,r16
	pop		r16
	ret	
	
;-----------------------------------------------
;setzt die eeprom- und ramadressen fr tsoll

setup_tsoll_adr:
sca16_1:
	ldi		r19,2				;2 byte (tsoll)
sca16_2:
	ldi		r24,low(ee_tsoll)	;ee-adr. brckensollstrom 
	ldi		r25,high(ee_tsoll)
	ldi		zl,low(tsoll)		;ram-adr. brckensollstrom 
	ldi		zh,high(tsoll)
	ret
;---------------------------------------------------
;schreibt tsoll -> eeprom
save_tsoll:
	push	zh
	push	zl
	push	yh
	push	yl
	push	r19
	rcall	setup_tsoll_adr
	rcall	ram_2_ee			;tsoll ram -> ee
	pop		r19
	pop		yl
	pop		yh
	pop		zl
	pop		zh
	ret	
	
;------------------------------------------------
;holt tsoll aus dem eeprom ins ram
get_tsoll:
	push	zh
	push	zl
	push	yh
	push	yl
	push	r19
	rcall	setup_tsoll_adr
	rcall	ee_2_ram			;tsoll ee -> ram
	pop		r19
	pop		yl
	pop		yh
	pop		zl
	pop		zh
	ret


; ======================================================
;schreibt Soll: ; Ist: ;  C ; -> LCD 
SollIstC:
	lcd_coll 0
	lcd_page 0
	ldi		zh,high(txt10<<1)	;"SOLL:"
	ldi		zl,low(txt10<<1)
	rcall	stringf
;	ret
	lcd_coll 0
	lcd_page 3
	ldi		zh,high(txt20<<1)	;"IST :"
	ldi		zl,low(txt20<<1)
	rcall	stringf
;	ret
	lcd_coll 104
	lcd_page 0
	ldi		zh,high(txt40<<1)	;"C"
	ldi		zl,low(txt40<<1)
	rcall	stringf
;	ret
	lcd_coll 104
	lcd_page 3
	ldi		zh,high(txt40<<1)	;"C"
	ldi		zl,low(txt40<<1)
	rcall	stringf
	ret
	


txt10:	.db	"SOLL:",0
txt20:	.db "IST :",0
;txt30:	.db	"375",0
txt40:	.db "C",0,0
txt50:	.db "LTKOLBEN?",0,0

;---------------------------------------------------
;schreibt den istwert -> lcd
ist2lcd:
	lcd_coll 65
	lcd_page 3
	ldi		zh,high(bcdi+2)
	ldi		zl,low(bcdi+2)
	rcall	stringr
	ret
;---------------------------------------------------
;schreibt den sollwert -> lcd
soll2lcd:
	lcd_coll 65
	lcd_page 0
	ldi		zh,high(bcds+2)
	ldi		zl,low(bcds+2)
	rcall	stringr
	ret

;---------------------------------------------------
;gibt eine mit null abgeschlossene zeichenkette 
;groer zeichen (12x16 Pixel) aus
;zu beginn: z = adresse zeichenkette im flash

stringf:
	push	r16
strf10:
	lpm		r16,z+			;holt zeichen
	tst		r16				;test ob ende (null)
	breq	strf20
	rcall	char_out		;zeichen -> lcd-display
	lds		r16,Lcoll		;
	subi	r16,-13			;breite eines zeichens (12+1)
	sts		Lcoll,r16
	rcall	lcd_set_coll	;setzt spalte neu
	rcall	lcd_set_page	;setzt seite neu
	rjmp	strf10
strf20:
	pop		r16
	ret
;---------------------------------------------------
;gibt eine mit null abgeschlossene zeichenkette 
;groer zeichen (12x16 Pixel) aus
;zu beginn: z = adresse zeichenkette im ram

stringr:
	push	r16
strr10:
	ld		r16,z+			;holt zeichen
	tst		r16				;test ob ende (null)
	breq	strr20
	rcall	char_out		;zeichen -> lcd-display
	lds		r16,Lcoll		;
	subi	r16,-13			;breite eines zeichens (12+1)
	sts		Lcoll,r16
	rcall	lcd_set_coll	;setzt spalte neu
	rcall	lcd_set_page	;setzt seite neu
	rjmp	strr10
strr20:
	pop		r16
	ret

;----------------------------------------------------
;gibt ein 12x16 pixel zeichen an der vorher gesetzten
;spalte und seite
;zu beginn: r16 = zeichen (8-bit ascii)

char_out:
	push	r17
	push	r18
	push	r19
	push	zl
	push	zh
	ldi		zh,high(font12x16<<1)
	ldi		zl,low(font12x16<<1)	;start fonttable
	ldi		r19,font_chars			;anzahl der eintrge in font 
co10:
	lpm		r17,z
	cp		r16,r17
	breq	co20
	adiw	zl,25
	dec		r19
	brne	co10
	rjmp	bad_char
co20:
	adiw	zl,1
	push	zl
	push	zh
	ldi		r18,2
co30:
	ldi		r19,12
co40:
	lpm		r16,z
	rcall	lcd_send
	adiw	zl,2
	dec		r19
	brne	co40
	dec		r18
	breq	co50
	pop		zh
	pop		zl
	adiw	zl,1
	rcall	lcd_set_coll
	lds		r16,Lpage
	inc		r16
	rcall	lsp10
	rjmp	co30
co50:
	pop		zh
	pop		zl
	pop		r19
	pop		r18
	pop		r17
	ret


bad_char:				;zeichen nicht vorhanden
	pop		zh
	pop		zl
	pop		r19
	pop		r18
	pop		r17
	ret	




;----------------------------------------------------
;aus font2/12x16_vertikal_LSB_2.asm
font12x16:

	.db 'B',\
	 0x00,0x00,0xFF,0x3F,0xFF,0x3F,0xC3,0x30,0xC3,0x30,0xC3,0x30,\
	 0xC3,0x30,0xE7,0x30,0xFE,0x39,0xBC,0x1F,0x00,0x0F,0x00,0x00,\
	 'C',\
	 0x00,0x00,0xF0,0x03,0xFC,0x0F,0x0E,0x1C,0x07,0x38,0x03,0x30,\
	 0x03,0x30,0x03,0x30,0x07,0x38,0x0E,0x1C,0x0C,0x0C,0x00,0x00,\
	 'E',\
	 0x00,0x00,0xFF,0x3F,0xFF,0x3F,0xC3,0x30,0xC3,0x30,0xC3,0x30,\
	 0xC3,0x30,0xC3,0x30,0xC3,0x30,0x03,0x30,0x03,0x30,0x00,0x00,\
 	 'I',\
	 0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x30,0x03,0x30,0xFF,0x3F,\
	 0xFF,0x3F,0x03,0x30,0x03,0x30,0x00,0x00,0x00,0x00,0x00,0x00,\
	 'K',\
	 0x00,0x00,0xFF,0x3F,0xFF,0x3F,0xC0,0x00,0xE0,0x01,0xF0,0x03,\
	 0x38,0x07,0x1C,0x0E,0x0E,0x1C,0x07,0x38,0x03,0x30,0x00,0x00,\
	 'L',\
	 0x00,0x00,0xFF,0x3F,0xFF,0x3F,0x00,0x30,0x00,0x30,0x00,0x30,\
	 0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x00,\
	 'N',\
	 0x00,0x00,0xFF,0x3F,0xFF,0x3F,0x0E,0x00,0x38,0x00,0xF0,0x00,\
	 0xC0,0x03,0x00,0x07,0x00,0x1C,0xFF,0x3F,0xFF,0x3F,0x00,0x00,\
	 'O',\
	 0x00,0x00,0xF0,0x03,0xFC,0x0F,0x0E,0x1C,0x07,0x38,0x03,0x30,\
	 0x03,0x30,0x07,0x38,0x0E,0x1C,0xFC,0x0F,0xF0,0x03,0x00,0x00,\
	 'S',\
	 0x00,0x00,0x3C,0x0C,0x7E,0x1C,0xE7,0x38,0xC3,0x30,0xC3,0x30,\
	 0xC3,0x30,0xC3,0x30,0xC7,0x39,0x8E,0x1F,0x0C,0x0F,0x00,0x00,\
	 'T',\
	 0x00,0x00,0x00,0x00,0x03,0x00,0x03,0x00,0x03,0x00,0xFF,0x3F,\
	 0xFF,0x3F,0x03,0x00,0x03,0x00,0x03,0x00,0x00,0x00,0x00,0x00,\
	 '',\
	 0x00,0x00,0xE0,0x0F,0xF0,0x1F,0x3B,0x38,0x1B,0x30,0x18,0x30,\
	 0x18,0x30,0x1B,0x30,0x3B,0x38,0xF0,0x1F,0xE0,0x0F,0x00,0x00,\
	 '?',\
	 0x00,0x00,0x1C,0x00,0x1E,0x00,0x07,0x00,0x03,0x00,0x83,0x37,\
	 0xC3,0x37,0xE3,0x00,0x77,0x00,0x3E,0x00,0x1C,0x00,0x00,0x00,\
	 ':',\
	 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x70,0x1C,0x70,0x1C,\
	 0x70,0x1C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,\
	 ' ',\
	 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,\
	 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,\
	 '0',\
	 0x00,0x00,0xF8,0x07,0xFE,0x1F,0x06,0x1E,0x03,0x33,0x83,0x31,\
	 0xC3,0x30,0x63,0x30,0x33,0x30,0x1E,0x18,0xFE,0x1F,0xF8,0x07,\
     '1',\
	 0x00,0x00,0x00,0x00,0x00,0x00,0x0C,0x30,0x0C,0x30,0x0E,0x30,\
	 0xFF,0x3F,0xFF,0x3F,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x00,\
	 '2',\
	 0x00,0x00,0x1C,0x30,0x1E,0x38,0x07,0x3C,0x03,0x3E,0x03,0x37,\
	 0x83,0x33,0xC3,0x31,0xE3,0x30,0x77,0x30,0x3E,0x30,0x1C,0x30,\
	 '3',\
	 0x00,0x00,0x0C,0x0C,0x0E,0x1C,0x07,0x38,0xC3,0x30,0xC3,0x30,\
	 0xC3,0x30,0xC3,0x30,0xC3,0x30,0xE7,0x39,0x7E,0x1F,0x3C,0x0E,\
	 '4',\
	 0x00,0x00,0xC0,0x03,0xE0,0x03,0x70,0x03,0x38,0x03,0x1C,0x03,\
	 0x0E,0x03,0x07,0x03,0xFF,0x3F,0xFF,0x3F,0x00,0x03,0x00,0x03,\
	 '5',\
	 0x00,0x00,0x3F,0x0C,0x7F,0x1C,0x63,0x38,0x63,0x30,0x63,0x30,\
	 0x63,0x30,0x63,0x30,0x63,0x30,0xE3,0x38,0xC3,0x1F,0x83,0x0F,\
	 '6',\
	 0x00,0x00,0xC0,0x0F,0xF0,0x1F,0xF8,0x39,0xDC,0x30,0xCE,0x30,\
	 0xC7,0x30,0xC3,0x30,0xC3,0x30,0xC3,0x39,0x80,0x1F,0x00,0x0F,\
	 '7',\
	 0x00,0x00,0x03,0x00,0x03,0x00,0x03,0x00,0x03,0x30,0x03,0x3C,\
	 0x03,0x0F,0xC3,0x03,0xF3,0x00,0x3F,0x00,0x0F,0x00,0x03,0x00,\
	 '8',\
	 0x00,0x00,0x00,0x0F,0xBC,0x1F,0xFE,0x39,0xE7,0x30,0xC3,0x30,\
	 0xC3,0x30,0xC3,0x30,0xE7,0x30,0xFE,0x39,0xBC,0x1F,0x00,0x0F,\
	 '9',\
	 0x00,0x00,0x3C,0x00,0x7E,0x00,0xE7,0x30,0xC3,0x30,0xC3,0x30,\
	 0xC3,0x38,0xC3,0x1C,0xC3,0x0E,0xE7,0x07,0xFE,0x03,0xFC,0x00,\
	 '',\
	 0x00,0x00,0x00,0x00,0x00,0x00,0x1E,0x00,0x3F,0x00,0x33,0x00,\
	 0x33,0x00,0x3F,0x00,0x1E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0

.equ	font_chars	= 25	;anzahl der eintrge in font
; =============================================================================

;------------------------------------
;setup timer1 fr adw und triac-zndung

ini_timer1: 
	ldi		r16,0b00000001	;timer counter2 reset prescaler 
	out		GTCCR,r16
	clr		r16
	sts		TCNT1H,r16
	sts		TCNT1L,r16
	ldi		r17,0x0a		;alle 2ms ein int.
	ldi		r16,0x00		;      "
	sts		ICR1H,r17		;ICR1 = top
	sts		ICR1L,r16
	ldi		r17,0x09		;zndimpuls triac ca. 100us
	ldi		r16,0x80		;      "
	sts		OCR1BH,r17
	sts		OCR1BL,r16		;OCR1B zndimpulslnge
	ldi		r16,0b00100010	;timer1 fast pwm mode
	sts		TCCR1A,r16
	ldi		r16,0b00011000
	sts		TCCR1B,r16
	clr		r16
	sts		TCCR1C,r16
	ldi		r16,0b00100000	;timer1 icr1-int. ein
	sts		TIMSK1,r16
	ldi		r16,0b00011010	;timer mode 12, prescaler clk/8
	sts		TCCR1B,r16
	clr		adl
	clr		adh
	ldi		r16,pre_cnt		;einschwingzhler setzen
	mov		cnt_u,r16
	ldi		r16,0b00001000
	sts		EICRA,r16		;int1 falling edge
	ldi		r16,0b00000010
	out		EIMSK,r16		;int1 enable
	ret
;-------------------------------
;--------------------------------------------------
;schlatet den portpin pd.6 bei jedem aufruf um
;praktisch zur zeitmessung zwischen zwei aufrufe.

blink:
	sbi		pind,6
	ret


;-----------------------------------------------
;timer 1 overflow interrupt alle 2ms fr
;den software a/d-wandler.
;Uref wird eingeschaltet (V17),
;wenn der ausgang des integrators (N5) 
;nicht auf +u hngt

tim1_ov_int:
	push	r16
;	sbi		pind,6			;******test only
	cbi		portc,2			;ref ein
	sbis	pind,3			;test portd.3 1->0
	sbi		portc,2			;ref aus
	ldi		r16,0b00000010	;a.comp.int1, ein
	out		EIMSK,r16
	pop		r16
	reti

;-----------------------------------------------
;fr software a/d-wandler
;wenn der komparatorint.(N4_2) kommt, wird der 
;zhlerstand von timer 1 ausgelesen. Dies ist
;das ergebnis einer untermessung.
;dieser wert wird auf dem ergebniszhler solange
;aufaddiert, bis die anzahl der untermessungen
;erreicht ist. das endergebnis wird kanalnr.bezogen
;abespeichert und das fertigflag gesetzt. 
;
acomp_int:
	sbi		portc,2			;ref aus
	push	r17
	push	r16
	lds		r16,TCNT1L		;ergebnis einer untermessung
	lds		r17,TCNT1H		;   holen
	in		status,sreg		;statusreg. sichern
	push	r31
	push	r30
	ldi		r30,0b00000000	;comp.int, aus
	out		EIMSK,r30
aci1:
	sbrc	flag,ad_rdy		;test adw fertig??
	rjmp	aci4			;wenn fertig, dann nichts tun
	add		adl,r16
	adc		adh,r17			;ergebniszhler aktualisieren
	dec		cnt_u			;untermessungszhler -1
	brne	aci4			;test ob ende
	sbrc	flag,ad_m		;test einschwingen/hauptmessp.
	rjmp	aci3
	sbr		flag,(1<<ad_m)	;setzt flag(ad_m) hauptmessphase
	clr		adl				;setzt ergebniszhler=0
	clr		adh
	ldi		r16,u_cnt		;setzt anzahl der untermessungen
	mov		cnt_u,r16
	rjmp	aci4
aci3:
	ldi		r30,low(chn)	;adresse aktueller ad-kanal
	ldi		r31,high(chn)
	ld		r16,z+			;zeiger -> 1.a/d-wert
	add		r30,r16			
	eor		r16,r16			;berechnet zeiger ->
	adc		r31,r16			; aktuellen a/d-wert (kanal)
	st		z+,adl			;speichert a/d-wert (kanal)
	st		z,adh
	cbr		flag,(1<<ad_m)	;lscht flag hauptmessphase
	sbr		flag,(1<<ad_rdy);setzt flag, dass adw fertig
	ldi		r16,pre_cnt		;einschwingzhler setzen
	mov		cnt_u,r16	
aci4:
	pop		r30
	pop		r31
	pop		r16
	pop		r17
	out		sreg,status		;statusreg. wieder setzen

	reti
;------------------------------------------------
;adu1k		
;startet den adw und subtrahiert vom ergebnis
;den offset Uoff und korigiert den wert auf 1mV auflsung
;Kf = korrekturfaktor 
;am ende  : r17:r16 = (ad-wert - offset)x (Kf) / (0x10000)
;
;die korrektur ber lookup table ist besser


adu1k:
	push	r18
	push	r19
	push	r20
	push	r21
	push	r22
	push	r23
	rcall	adu1
	ldi		r20,low(Uoff1)
	ldi		r21,high(Uoff1)
	rcall	sub16				;subtrahiert den offset
	ldi		r18,low(Kf)
	ldi		r19,high(Kf)
	rcall	MUL16x16
	movw	r16,r22				;ergebnis in r23:r22
;	rcall	lr17r16_hex_crlf	;***test only
	pop		r23
	pop		r22
	pop		r21
	pop		r20
	pop		r19
	pop		r18
	ret

.equ	Kf	= 710		;J-Typ Thermoelement
;----------------------------------------------------------

;------------------------------------------------
;adu1		
;schaltet mux-kanal (r16) ein.
;startet den adw
;zu beginn: r16 = ad-kanal
;am ende  :cy=0, r17:r16 = ad-wert
;fehler -> cy=1, z.B. kein Ltkolben 
adu1:
	sts		chn,r16				;setzt ad-kanal
	rcall	out_chn
	andi	flag,0b11100011		;setzt ad-flags zurck
	clr		r17
adu2:
	dec		r17
	breq	adu3				;-> ad-rdy nicht gekommen
	rcall	del1ms
	sbrs	flag,ad_rdy			;testet ob adw fertig
	rjmp	adu2
	lds		r16,chn
	rcall	get_adw_r16			;holt den ad-wert
	rcall	lr17r16_hex_crlf		;***test only
	clc							;alles ok -> cy=0
	ret
adu3:
	sec							;fehler -> cy=1
	ret


;------------------------------------------------
;setzt die ad-kanaladresse fr mux D1
;zu beginn: r16 = kanaladresse
out_chn:
	push	r17
	in		r17,portb
	andi	r17,0b11111100
	or		r17,r16
	out		portb,r17
	pop		r17
	ret

;------------------------------------------------
;holt den a/d-wert dessen kanaladresse in r16 angegeben ist
;zu beginn: r16 = kanaladresse 
;am ende:
; -> r16 = low-a/d-wert
; -> r17 = high-a/d-wert

get_adw_r16:
	ldi		r28,low(ad_ch)	;adresse aktueller ad-kanal
	ldi		r29,high(ad_ch)
	add		r28,r16
	clr		r16			
;	eor		r16,r16			;berechnet zeiger ->
	adc		r29,r16			; low-a/d-wert
	ld		r16,y+			;low-a/d-wert  -> r16
	ld		r17,y			;high-a/d-wert -> r17
	ret



 
;-------------------------------------------------------------
;initialisiert timer2 alle 2ms timeroverflow-interrupt

ini_timer2:
	push	r16
	clr		r16
	sts		TCNT2,r16			;timer counter2=0
	ldi		r16,0b00000011		
	sts		TCCR2A,r16			;timer counter2 cntr2A=3 
	ldi		r16,0b00000001		;timer counter2 reset prescaler 
	out		GTCCR,r16
	ldi		r16,159				;alle 2msec int. 
	sts		OCR2A,r16		
	ldi		r16,0b00001101		;timer counter2 prescal/128 run
	sts		TCCR2B,r16
	ldi		r16,0b00000001		;timer counter2 overfl int en. 
	sts		TIMSK2,r16
	pop		r16		
	ret

;----------------------------------------------------
t2ms:
	sbi		pinc,2		;ref aus
	reti



;----------------------------------------------------
;testet die triaczndung
;komparator N4_1 hat am ausgang einen 50Hz rechteck (netzsyncron)
;nach einem wechsel wird nach 5ms ein 100us zndimpuls ausgegeben,
;dann wird 10ms gewartet und ein zweiter zndimpuls ausgegeben. 
;wenn alles ok ist erzeugt der triac einen 50%tigen Phasenanschnitt
; --> oszi
;test ok. 07.07.2020

chk_triac:
cktr10:
	sbic	pind,2
	rjmp	cktr10		;wartet bis 0
cktr20:
	sbis	pind,2
	rjmp	cktr20		;wartet bis 0 -> 1
	rcall	del5ms
	cbi		portb,2		;start zndimpuls
	rcall	del100us
	sbi		portb,2		;stop zndimpuls
	rcall	del10ms
	cbi		portb,2		;start zndimpuls
	rcall	del100us
	sbi		portb,2		;stop zndimpuls
	rjmp	cktr10	



;----------------------------------------------------
; sets the output stream to either null/serial/ram
stream_1:
	push	temp1
	ldi		temp1,lcd_stream
	sts		stream_destination,temp1
	pop		temp1
	ret
;------------------------------------------------
;gibt die ersten 512 bytes des internen rams zur
;console aus

ram_dump:
	push	r16
	push	r17
	push	r18
	push	r19
	push	zl
	push	zh
	sbr		flag,(1<<flg_co) ;ausgabe -> console
	rcall	crlf
	ldi		zl,low(0x100)	;start adresse ram
	ldi		zh,high(0x100)
	ldi		r18,32			;16 zeilen
ramd1:
	rcall	crlf
	ldi		r19,16			;zeichen/zeile
ramd2:
	mov		r16,zh
	rcall	lbyte			;adresse ausgeben
	mov		r16,zl
	rcall	lbyte			;adresse ausgeben
	ldi		r16,':'
	rcall	co
ramd3:
	rcall	blk
	ld		r16,z+			;ein byte lesen
	rcall	lbyte
;	adiw	r24,1			;adresse +1
	dec		r19
	brne	ramd3
	dec		r18
	brne	ramd1
	rcall	crlf
	pop		zh
	pop		zl
	pop		r19
	pop		r18
	pop		r17
	pop		r16
	ret


;-----------------------------------------------------		
;************************************************
;***********************************************
;diverse routinen fr das interne eeprom (512 byte)
;-----------------------------------------------


;-----------------------------------------------
;schreibt 1 byte ins eeprom (3,4 msec)
;zu beginn: r25:r24 = adresse im eeprom
;               r16 = byte zu schreiben
ee_erase_write:
	push	r18
eeew1:
	sbic	EECR,EEPE		;falls schreibvorgang
	rjmp	eeew1			;  noch aktiv -> warten
	clr		r18
	out		eecr,r18		
	out		eearl,r24		;adresse setzen
	out		eearh,r25		;adresse setzen
	out		eedr,r16		;daten ausgeben
	cli						;interrupts verbieten
	sbi 	eecr,eempe		;set master write enable
	sbi		eecr,eepe		;eeprom write strobe
	sei						;interrupts erlauben
	pop		r18
	ret	
;-----------------------------------------------
;lscht 1 byte im eeprom (1,8 msec)
;zu beginn: r25:r24 = adresse im eeprom


ee_erase_only:
	push	r16
eeeo1:
	sbic	EECR,EEPE		;falls schreibvorgang
	rjmp	eeeo1			;  noch aktiv -> warten
	ldi		r16,0x10		;erase only
	out		eecr,r16		
	out		eearl,r24		;adresse setzen
	out		eearh,r25		;adresse setzen
	cli						;interrupts verbieten
	sbi 	eecr,eempe		;set master write enable
	sbi		eecr,eepe		;eeprom write strobe
	sei						;interrupts erlauben
	pop		r16
	ret		
	
		
;-----------------------------------------------
;schreibt 1 byte ins eeprom (1,8 msec)
;byte muss vorher gelscht (0xff) sein
;es lassen sich auch einzelne bits rcksetzen (1 -> 0)
;0 -> 1 geht nicht!!
;zu beginn: r25:r24 = adresse im eeprom
;               r16 = byte zu schreiben

ee_write_only:
	push	r18
eewo1:
	sbic	EECR,EEPE		;falls schreibvorgang
	rjmp	eewo1			;  noch aktiv -> warten
	ldi		r18,0x20		;write only
	out		eecr,r18		
	out		eearl,r24		;adresse setzen
	out		eearh,r25		;adresse setzen
	out		eedr,r16		;daten ausgeben
	cli						;interrupts verbieten
	sbi 	eecr,eempe		;set master write enable
	sbi		eecr,eepe		;eeprom write strobe
	sei						;interrupts erlauben
	pop		r18
	ret		

;-----------------------------------------------
;liest 1 byte aus dem eeprom
;zu beginn: r25:r24 = adresse im eeprom
;am ende  : r16     = byte zu lesen
ee_read:
	sbic	EECR,EEPE		;falls schreibvorgang
	rjmp	ee_read			;  noch aktiv -> warten
	out		eearl,r24		;adresse setzen
	out		eearh,r25		;adresse setzen
	sbi		eecr,eere		;eeprom read strobe
	in		r16,eedr		;daten lesen
	ret		
;------------------------------------------------
;gibt die 512 bytes des internen eeproms zur
;console aus

ee_dump:
	push	r16
	push	r17
	push	r18
	push	r19
	push	r24
	push	r25
	sbr		flag,(1<<flg_co) ;ausgabe -> console
	rcall	crlf
	clr		r24				;start adresse = 0
	clr		r25
	ldi		r18,32			;16 zeilen
eed1:
	rcall	crlf
	ldi		r19,16			;zeichen/zeile
eed2:
	mov		r16,r25
	rcall	lbyte			;adresse ausgeben
	mov		r16,r24
	rcall	lbyte			;adresse ausgeben
	ldi		r16,':'
	rcall	co
eed3:
	rcall	blk
	rcall	ee_read			;ein byte lesen
	rcall	lbyte
	adiw	r24,1			;adresse +1
	dec		r19
	brne	eed3
	dec		r18
	brne	eed1
	rcall	crlf
	pop		r25
	pop		r24
	pop		r19
	pop		r18
	pop		r17
	pop		r16
	ret
;------------------------------------------------
;schreibt die forlaufende adresse (0 - 255)
;in die 512 bytes des internen eeproms

ee_fill_adr:
	push	r16
	push	r24
	push	r25
	clr		r24				;start adresse = 0
	clr		r25
eefa1:
	mov		r16,r24			;adresse -> r16 (daten)
	rcall	ee_write_only	;ein byte schreiben
;	rcall	lbyte
	adiw	r24,1			;adresse +1
	cpi		r25,2
	brne	eefa1
	pop		r25
	pop		r24
	pop		r16
	ret
;------------------------------------------------
;lscht (=> 0xFF) die 512 bytes des internen eeproms

ee_erase_all:
	push	r24
	push	r25
	clr		r24				;start adresse = 0
	clr		r25
eeea1:
	rcall	ee_erase_only	;ein byte lschen
;	rcall	lbyte
	adiw	r24,1				;adresse +1
	cpi		r25,2
	brne	eeea1
	pop		r25
	pop		r24
	ret
;------------------------------------------------
;liest bytes aus dem eeprom und schreibt diese ins ram
;zu beginn: r25:24 startadresse im eeprom
;           zh:zl  startadresse ram
;			r19	anzahl zu bertragender bytes
ee_2_ram:
	push	r16
ee2r_1:
	rcall	ee_read		;byte lesen aus eeprom
	st		z+,r16		;byte schreiben -> ram
	adiw	r24,1
	dec		r19
	brne	ee2r_1
	pop		r16
	ret
;------------------------------------------------
;liest bytes aus dem ram und schreibt diese ins eeprom
;zu beginn: r25:24 startadresse im eeprom
;           zh:zl  startadresse ram
;			r19	anzahl zu bertragender bytes
ram_2_ee:
	push	r16
r2ee_1:
	ld		r16,z+			;byte lesen aus <- ram
	rcall	ee_erase_write	;byte schreiben -> eeprom
	adiw	r24,1
	dec		r19
	brne	r2ee_1
	pop		r16
	ret

	


;************************************************


;------------------------------------------------
;sendet 8 bit ueber die spi-schnittstelle
;muss zu fuss gemacht werden, da debugwire
;spi abschaltet.
;zu beginn: r16 = zu sendendes byte
;
;pb3 ->  mosi (input)
;pb4 ->  miso (output)
;pb5 ->  sck  (output)
;
;ms-bit zuerst 
;spi_out
;
spi_out:
	push	r16
	push	r17
	ldi		r17,8		;8 bit
so2:
	cbi		portb,5		;sck start with 0
	rol		r16			;msb -> cy
	brcs	so3
	cbi		portb,4		;clear databit = 0
	rjmp	so4
so3:
	sbi		portb,4		;set databit = 1
so4:
	nop
	nop
	sbi		portb,5		;sck = 1
	nop
	nop
	dec		r17
	brne	so2
	pop		r17
	pop		r16
	ret

;------------------------------------------------
;empfngt 8 bit ueber die spi-schnittstelle
;muss zu fuss gemacht werden, da debugwire
;spi abschaltet.
;am ende: r16 = empfangenes byte
;
;
;pb3 ->  mosi (input)
;pb4 ->  miso (output)
;pb5 ->  sck  (output)
;
;
;ms-bit zuerst 
;spi_in
;
spi_in:
	push	r17
	ldi		r17,8		;8 bit
	clr		r16
	clc
si2:
	cbi		portb,5		;sck start with 0
	nop
	nop
	rol		r16			;msb -> cy
	sbic	pinb,3		;check inputpin	
	ori		r16,1		;set bit = 1
	nop
	nop
	sbi		portb,5		;sck = 1
	nop
	nop
	dec		r17
	brne	si2
	pop		r17
	ret
;--------------------------------------------------
;------------------------------------------------
;deaktiviert den Watchdog
wdt_off:
	wdr
	in		r16,MCUSR
	andi	r16,(0xff & (0<<WDRF))
	out		MCUSR,r16
	lds		r16,WDTCSR
	ori		r16,(1<<WDCE) | (1<<WDE)
	sts		WDTCSR,r16
	ldi		r16,(0<<WDE)
	sts		WDTCSR,r16
	ret



;--------------------------------------------------
;schlatet den portpin pd.5 (led backlight) bei jedem aufruf um
;praktisch zur zeitmessung zwischen zwei aufrufe.

blink_pd5:
	sbic	portd,5
	rjmp	blink5
	sbi		portd,5
	ret
blink5:
	cbi		portd,5
	ret


;-------------------------------------------------------
;schiebt r16 5 x rechts (/32) und testet ob r16=0
lsr5_r16:
	push	r17
	ldi		r17,5
lsr5_1:
	lsr		r16
	dec		r17
	brne	lsr5_1
;	rcall	lbyte
;	rcall	crlf
	tst		r16
	pop		r17
	ret
;-------------------------------------------------------
;schiebt r17:r16 6 x rechts (/64) und testet ob r17:r16=0
lsr6_r17r16:
	push	r19
	ldi		r19,6
lsr6b_1:
	lsr		r17
	lsr		r16
	dec		r19
	brne	lsr6b_1
;	rcall	lr17r16_hex_crlf	;***gibt r17:r16 in hex aus				;
	or		r16,r17		;test ob 0000
	pop		r19
	ret







	
;------------------------------------------------
;lscht (00) alle variablen von chn bis ds_end
clr_var:
	push	r16
	push	r17
	clr		r16			
	ldi		r17,((ds_end - chn)+1)
	ldi		zl,low(chn)		;startadr. chn
	ldi		zh,high(chn)
clrv1:
	st		z+,r16
	dec		r17
	brne	clrv1
	pop		r17
	pop		r16
	ret



;------------------------------------------------		
;------------------------------------------------
;************************************************
;*   diverse Mathe-Routinen                     *
;************************************************
;------------------------------------------------
;addiert zwei 16-bit werte
;zu beginn: r17:r16 = wert1 (r16 = low-byte) (r17 = high-byte)
;         : r21:r20 = wert2 (r20 = low-byte) (r21 = high-byte)
;am ende  : r17:r16 = ((r17:r16)+(r21:r20))
add16:
	add		r16,r20
	adc		r17,r21
	ret
;------------------------------------------------
;vergleicht einen wert in r17:r16 
;mit einem wert in r21:r20
;cy = 1,  r17:r16  < r21:r20
;cy = 0,  r17:r16 >= r21:r20
;r17:r16, r21:r20 werden nicht verndert
cp16:
	cp		r16,r20
	cpc		r17,r21
	ret
;------------------------------------------------
;vergleicht einen wert in r17:r16
;mit einem wert in r25:r24
;cy = 1,  r17:r16  < r25:r24
;cy = 0,  r17:r16 >= r25:r24
;r17:r16, r25:r24 werden nicht verndert
cp16b:
	cp		r16,r24
	cpc		r17,r25
	ret


;------------------------------------------------
;subtrahiert zwei 16-bit wert
;zu beginn: r17:r16 = wert1
;         : r21:r20 = wert2
;am ende  : r17:r16 = (r17:r16)-(r21:r20)
sub16:
	sub		r16,r20
	sbc		r17,r21
	ret

;-------------------------------------------------
;multipliziert einen 16 bit-wert x 256
;zu beginn: r17, r16            16-bit wert 
;am Ende  : r23, r22, r21, r20  Ergebnis

mul256:
	clr		r20
	clr		r23
	mov		r21,r16
	mov		r22,r17
	ret

;-------------------------------------------------
;Multiplying two 16-bit numbers can result in a four-byte result. 
;We use the hardware multiply (MUL) command to create all four
; cross products and add them to the 32-bit result. 
;The MUL command leaves it results each time in R1:R0 which we 
;then add into our result. The routine takes about twenty cycles.
;zu beginn: r17, r16            Multiplikant 
;			r19, r18            Multiplikator
;am Ende  : r23, r22, r21, r20  Ergebnis



MUL16x16:
        CLR 	R22             ;Set high bytes of result to zero            
        CLR 	R23             ;
        LDI 	R24,16          ;Bit Counter
MUL16_1:   
		LSR 	R19             ;Shift Multiplier to the right
        ROR 	R18             ;Shift lowest bit into Carry Flag
         BRCC 	MUL16_2            ;If carry is zero skip addition 
        ADD 	R22,R16         ;Add Multiplicand into high bytes
        ADC 	R23,R17         ;of the Result
MUL16_2:   
		ROR 	R23             ;Rotate high bytes of result into
        ROR 	R22             ;the lower bytes
        ROR 	R21             ;
        ROR 	R20             ;
        DEC 	R24             ;Check if all 16 bits handled
        BRNE 	MUL16_1            ;If not then loop back
		RET
;----------------------------------------------------------
;DIVIDING TWO 16-BIT NUMBERS
;zu beginn: r17:r16            Dividend 
;			r19:r18            Divisor
;am ende:   r1:r0			   Ergebnis
;			r3:r2              Rest

DIV1616:
		PUSH	R20
        MOVW 	R1:R0,R17:R16 	;Copy dividend into answer
        LDI 	R20,17          ;Load bit counter
        SUB 	R2,R2     		;Clear Remainder and Carry
        CLR 	R3          	;
DIV16_1:
   		ROL 	R0          	;Shift the answer to the left
        ROL 	R1          	;
        DEC 	R20         	;Decrement Counter
        BREQ 	DONE        	;Exit if sixteen bits done
        ROL 	R2          	;Shift remainder to the left
        ROL 	R3          	;
        SUB 	R2,R18       	;Try to subtract divisor from remainder
        SBC 	R3,R19
        BRCC 	SKIP        	;If the result was negative then
        ADD 	R2,R18       	;reverse the subtraction to try again
        ADC 	R3,R19       	;
        CLC               		;Clear Carry Flag so zero shifted into A 
        RJMP 	DIV16_1        ;Loop Back
SKIP:   
		SEC               		;Set Carry Flag to be shifted into A
        RJMP 	DIV16_1
DONE:
		POP		R20
		RET

;-----------------------------------------------------
;DIVIDING A 32-BIT NUMBER by 16 Bit
;The routine takes about 700 cycles.
;zu beginn: R19, R18, R17, R16 Dividend
;			R21, R20           Divisor
;Am Ende  : R3, R2, R1, R0     Ergebnis
;         : R7, R6, R5, R4     Rest

DIV3216:
		PUSH	R22
		PUSH	R8
        CLR 	R8
        MOVW 	R1:R0,R17:R16 	;Copy dividend into answer
        MOVW 	R3:R2,R19:R18 	;
        LDI 	r22,33          ;Load bit counter
        SUB 	R4,R4     		;Clear Remainder and Carry
        CLR 	R5          	;
        CLR 	R6          	;
        CLR 	R7          	;
DIV3216_1:   
		ROL 	R0          	;Shift the answer to the left
        ROL 	R1          	;
        ROL 	R2          	;
        ROL 	R3          	;
        DEC 	r22             ;Decrement Counter
        BREQ 	DIV3216_3        	;Exit if 32 bits done
        ROL 	R4          	;Shift remainder to the left
        ROL 	R5          	;
        ROL 	R6          	;
        ROL 	R7          	;
        SUB 	R4,r20       	;Try to subtract divisor from remainder
        SBC 	R5,r21       	;
        SBC 	R6,R8     		;
        SBC 	R7,R8     		;
        BRCC 	DIV3216_2        	;If the result was negative then
        ADD 	R4,r20       	;reverse the subtraction to try again
        ADC 	R5,r21       	;
        ADC 	R6,R8     		;
        ADC 	R7,R8     		;
        CLC               		;Clear Carry Flag so zero shifted into A 
        RJMP 	DIV3216_1        	;Loop Back
DIV3216_2:   
		SEC               		;Set Carry Flag to be shifted into A
        RJMP 	DIV3216_1
DIV3216_3:
		POP		R8
		POP		R22
		ret

;**********************************************************
;ab hier, diverse usart-programme (ein-, ausgaben)
;------------------------------------------------
;lr17r16_hex
;gibt (r17,r16) in hex ber die
;serielle schnittstelle aus

lr17r16_hex:
	push	r16
;	ldi		r16,0x30
;	rcall	co
	mov		r16,r17		;high-byte zuerst
	rcall	lbyte
	pop		r16			;dann low-byte
	rcall	lbyte
	rjmp	blk
;------------------------------------------------
;lr17r16_cr
;gibt (r17,r16) in hex ber die
;serielle schnittstelle und anschliessend
;cr aus (fr dasylab)

lr17r16_hex_cro:
	rcall	lr17r16_hex
	rjmp	cr
;------------------------------------------------
;lr17r16_blk
;gibt (r17,r16) in hex ber die
;serielle schnittstelle und anschliessend
;blank aus

lr17r16_hex_blk:
	rcall	lr17r16_hex
	rjmp	blk		
			



;------------------------------------------------
;lr17r16_crlf
;gibt (r17,r16) in hex ber die
;serielle schnittstelle und anschliessend
;crlf aus

lr17r16_hex_crlf:
	rcall	lr17r16_hex
	rjmp	crlf
			
;------------------------------------------------
;lr21r20_hex_crlf
;gibt (r21:r20) in hex ber die
;serielle schnittstelle und anschliessend
;crlf aus

lr21r20_hex_crlf:
	push	r17
	push	r16
	movw	r16,r20
	rcall	lr17r16_hex_crlf
	pop		r16
	pop		r17
	ret
			
;------------------------------------------------
;lr21r20_hex_blk
;gibt (r21:r20) in hex ber die
;serielle schnittstelle und anschliessend
;1 x blank aus

lr21r20_hex_blk:
	push	r17
	push	r16
	movw	r16,r20
	rcall	lr17r16_hex
	rcall	blk
	pop		r16
	pop		r17
	ret		



;-----------------------------------------------
; gibt r16 und 17 aus.
;anschliessend 1 x cr  (fuer dasylab)
tx_r17r16_bin:
	push	r16
	rcall	co				;low-byte ausgeben
	mov		r16,r17
	rcall	co				;high-byte ausgeben
	ldi		r16,0x0d
	rcall	co
	pop		r16
	ret


;----------------------------------------------------------
;holt byte aus adresse (Z), erhoeht adresse Z
;und zeigt byte in hex ueber uart-port an.
;wiederholt den vorgang bis r17 = 0.
;zu beginn: r17 = schleifenzaehler
;			Z   = Startadresse
		
lreg:
	ld		r16,Z+
	rcall	lbyte
	rcall	blk
	dec		r17
	brne	lreg
	ret


;----------------------------------------------------------
;lbyte
;gibt byte in r16 als hex-wert ueber uart aus. 

lbyte:
	push	r16			;rette byte
	push	r16			;rette byte
	swap	r16			;vertausche obere und untere 4 bit
	rcall	hxd			;wandle unteren 4 bits -> hex
	pop		r16			;stelle das byte wieder her
	rcall	hxd			;wandle obere 4 bits -> hex
	pop		r16			;stelle das byte wieder her
	ret

;----------------------------------------------------------
hxd:
	rcall	conv		;convertiert nibble -> hex
	rjmp	co			;und gibt ueber portpin aus

;----------------------------------------------------------
; conv
;convertiert 4 bit bin-wert -> ascii char.
;input  reg r16 = 0....fh
;output reg r16 = 30h....39h,  41h....46h
	
conv:				;                             		
	andi	r16,0x0f	; bit 7...4 = 0
	subi	r16,-'0'	;addiere ASCII 0               		
	cpi		r16,'9'+1	;ziffern A...F?                		
	brcs	conv1		; --> Yes (A to F set)        		
	subi	r16,-7		;addiere 7 fuer A...F
conv1:
	ret
					;                             		
;----------------------------------------------------------
;----------------------------------------------------------
; "blkp"
; ausgabe 1 x leerzeichen ueber uart

blk:
	push	r16
	ldi		r16,0x20
	rcall	co
	pop		r16
	ret

;----------------------------------------------------------
; "cro"
; ausgabe 1 x cr ueber uart

cro:
	push	r16
	ldi		r16,0x0d
	rcall	co
;	rcall	del1ms		;fuer hyperterminal
	pop		r16
	ret


;----------------------------------------------------------
; "crlf"
; ausgabe 1 x crlf ueber usart

crlf:
	push	r16
	ldi		r16,0x0d
	rcall	co
	ldi		r16,0x0a
	rcall	co
;	rcall	del1ms		;fuer hyperterminal
	pop		r16
	ret

;----------------------------------------------------------
;lbit
;gibt byte in r16 als folge von 0 und 1 ueber usart an.
;zu beginn: r16 = byte

lbit:
	push	r16
	push	r17
	push	r18
	mov		r18,r16
	ldi		r17,8
lbp1:
	ldi		r16,'0'
	rol		r18
	brcc	lbp2
	inc		r16
lbp2:
	rcall	co
	rcall	blk
	dec		r17
	brne	lbp1
	pop		r18
	pop		r17
	pop		r16
	ret


;----------------------------------------------------------
; "uartp_out"
; software-uart
; This subroutine transmits the byte stored in r16
; over port-pin
; Baud Rate				: 38400 Bd /8/N/1/
; TX-port				: PORTB.4 (ATmega168;MISO, pin16)
;del -> Bd
;8us = 115200 Bd
;26us = 38400 Bd 

uartp_out:
co:
	push	r16
	push	r17
	ldi		r17,10		;1+8+sb (sb is # of stop bits)
	com		r16			;Inverte everything
	sec					;Start bit
upo0:
	brcc	upo1		;If carry set
	cbi		PORTB,4		;    send a '1'
	rjmp	upo2		;else	
upo1:	
	sbi		PORTB,4		;    send a '0'
upo2:

	rcall 	del26us		;One bit delay
	lsr		r16			;Get next bit
	dec		r17			;If not all bit sent
	brne	upo0		;   send next
;	rcall 	del100us		; delay
	pop		r17
	pop		r16			;
	ret					;   return
	

;------------------------------------------------
;------------------------------------------------
;prnts
;2.) ausgabe einer mit 0 abgeschlossenen 
;zeichenkette (asciiz) zur Console
;start : z = startadresse
;ende  : z = startadresse

prnts:
	push	r16
	push	r17
	push	zl
	push	zh
prnts1:
	lpm		r16,z+			;holt zeichen
	tst		r16				;test ob ende (=0)
	breq	prnts2
	rcall	co				;zeichen -> console
	rjmp	prnts1
prnts2:
	pop		zh
	pop		zl
	pop		r17
	pop		r16
	ret

crlf_prnts:
	rcall	crlf
	rjmp	prnts
;------------------------------------------------------
; Bin2ToBcd5
; ==========
; converts a 16-bit-binary to a 5-digit-BCD
; zu beginn: 16-bit-binary in r17:r16
;          : z = zieladresse im ram (0 abgeschlossen)        
; Out: 5-digit-BCD, output -> ram (0 abgeschlossen)
; Used registers: r17:r16 (unchanged), r19:r18 (unchanged),
; r21:r20 (unchanged),
;
; Called subroutines: Bin2ToDigit
;
Bin2ToBcd5:
	push	r21
	push	r20
	push	r19
	push	r18
	push 	r17 			; Save number
	push 	r16
	movw	r18,r16
	ldi 	r16,HIGH(10000) ; Start with tenthousands
	mov 	r21,r16
	ldi 	r16,LOW(10000)
	mov 	r20,r16
	rcall 	Bin2ToDigit 	; Calculate digit
	ldi 	r16,HIGH(1000) 	; Next with thousands
	mov 	r21,r16
	ldi 	r16,LOW(1000)
	mov 	r20,r16
	rcall 	Bin2ToDigit 	; Calculate digit
	ldi 	r16,HIGH(100) 	; Next with hundreds
	mov 	r21,r16
	ldi 	r16,LOW(100)
	mov 	r20,r16
	rcall 	Bin2ToDigit 	; Calculate digit
	ldi 	r16,HIGH(10) 	; Next with tens
	mov 	r21,r16
	ldi 	r16,LOW(10)
	mov 	r20,r16
	rcall 	Bin2ToDigit 	; Calculate digit
	mov 	r16,r18 		; Remainder are ones
	subi	r16,0xd0		; convert -> ascii (adi 0x30)
	st		z+,r16			;bcd-digit save to ram
	clr		r16
	st		z,r16			;terminate bcd-sting with zero		
;	rcall	co 				; digit -> console
	pop 	r16 			; Restore original binary
	pop 	r17
	pop		r18
	pop		r19
	pop		r20
	pop		r21
	ret 					; and return
;----------------------------------------------------
; Bin2ToDigit
; ===========
; converts one decimal digit by continued subraction of a
;   binary coded decimal
; Used by: Bin2ToBcd5, Bin2ToAsc5, Bin2ToAsc
; In: 16-bit-binary in r19:L, binary coded decimal in
;   r21:L, Z points to current BCD digit
; Out: Result in Z, Z incremented
; Used registers: r19:L (holds remainder of the binary),
;   r21:L (unchanged), r16
; Called subroutines: -
;
Bin2ToDigit:
	clr 	r16 			; digit count is zero
Bin2ToDigita:
	cp 		r19,r21 		; Number bigger than decimal?
	brcs 	Bin2ToDigitc 	; MSB smaller than decimal
	brne 	Bin2ToDigitb 	; MSB bigger than decimal
	cp 		r18,r20 		; LSB bigger or equal decimal
	brcs 	Bin2ToDigitc 	; LSB smaller than decimal
Bin2ToDigitb:
	sub 	r18,r20 		; Subtract LSB decimal
	sbc 	r19,r21 		; Subtract MSB decimal
	inc 	r16 			; Increment digit count
	rjmp 	Bin2ToDigita 	; Next loop
Bin2ToDigitc:
	subi	r16,0xd0		;convert -> ascii
	st		z+,r16			;bcd-digit save to ram		
;	rcall	co 				; digit -> console
	ret 					; done
;


;----------------------------------------------------
; AscToBin2
; =========
; converts an ASCII coded number to a 2-Byte bi-
; nary
; In: Z points to first digit, conversion stops at first
;   digit detected or if overflow of the result occurs,
;   end of number must be terminated by a non-decimal
;   ASCII-digit!
; Out: Z points to first non-valid digit or to the digit
;   where the overflow occurred, if number is valid the
;   T-Flag is clear and the number is in registers
;   r21:r20
; Used registers: r21:L (result), r23:L (restored
;   after use), r16
; Called subroutines: Bin1Mul10
;
AscToBin2:
	clr 	r21 		; Clear the result
	clr 	r20
	clt 				; Clear error flag bit
AscToBin2a:
	ld 		r16,Z+ 		; ignore leading blanks and zeros
	cpi 	r16,' ' 	; blank?
	breq 	AscToBin2a
	cpi 	r16,'0' 	; zero?
	breq 	AscToBin2a
AscToBin2b:
	subi 	r16,'0' 	; subtract ASCII zero
	brcs 	AscToBin2d 	; End of the number
	cpi 	r16,10 		; check invalid digit
	brcc 	AscToBin2d 	; No-decimal char
	rcall 	Bin1Mul10 	; Multiply binary number by 10
	brts 	AscToBin2c 	; overflow, return with T-Flag set
	add 	r20,r16 	; add the digit to the binary
	ld 		r16,Z+ 		; read next char
	brcc 	AscToBin2b 	; no overflow to binary MSB
	inc 	r21 		; Overflow to binary MSB
	brne 	AscToBin2b 	; no overflow of binary MSB
	set 				; Set overflow flag
AscToBin2c:
	sbiw 	ZL,1 		; Back one char, last char end/invalid
AscToBin2d:
	ret

;--------------------------------------------------------
; Bin1Mul10
; =========
; multiplies a 16-bit-binary by 10
; Sub used by: AscToBin2, Asc5ToBin2, Bcd5ToBin2
; In: 16-bit-binary in r21:L
; Out: T-flag shows general result:
;   T=0: Valid result, 16-bit-binary in r21:L ok
;   T=1: Overflow occurred, number too big
;
Bin1Mul10:
	push 	r23 		; Save the register of 16-bit-binary 2
	push 	r22
	mov 	r23,r21 	; Copy the number
	mov 	r22,r20
	add 	r20,r20 	; Multiply by 2
	adc 	r21,r21
	brcs 	Bin1Mul10b 	; overflow, get out of here
Bin1Mul10a:
	add 	r20,r20 	; again multiply by 2 (4*number reached)
	adc 	r21,r21
	brcs 	Bin1Mul10b 	; overflow, get out of here
	add 	r20,r22 	; add the copied number (5*number reached)
	adc 	r21,r23
	brcs	 Bin1Mul10b ;overflow, get out of here
	add 	r20,r20 	; again multiply by 2 (10*number reached)
	adc 	r21,r21
	brcc 	Bin1Mul10c 	; no overflow occurred, don't set T-flag
Bin1Mul10b:
	set 				; an overflow occurred during multplication
Bin1Mul10c:
	pop 	r22 		; Restore the registers of 16-bit-binary 2
	pop 	r23
	ret
;

;------------------------------------------------

;************************************************
;ab hier, diverse testprogramme
;------------------------------------------------
;---------------------------------------------------

;chk_ue
;fuehrt staendig eine a/a-wandlung
;des ad-kanal7 aus und gibt das ergebnis
;in hex ueber die serielle schnittstelle aus


;------------------------------------------------
;wechselt die polaritt von port b1

tog_pb1:
	sbi		portb,1
	rcall	del100us
	cbi		portb,1
	rcall	del100us
	rjmp	tog_pb1
;------------------------------------------------
;wechselt die polaritt von port d5

tog_pd5:
	sbi		portd,5
	rcall	del1ms
	cbi		portd,5
	rcall	del1ms
	rjmp	tog_pd5


;------------------------------------------------
;testet das senden der seriellen schnittstelle co
;test mit 57600 Bd ok.
ser_tst:

st1:
	ldi		r16,0x55
	rcall	co
;	inc		r16
	rjmp	st1








;------------------------------------------------
;wandelt wert in r17:r16 -> bcd -> console
;1 x crlf
;1 x 200msec delay

b2d5_crlf:
	rcall	Bin2ToBcd5		;entladestrom -> console
	rcall	crlf
	rjmp	del200ms


;------------------------------------------------
;************************************************
;ab hier, diverse debugprogramme
;------------------------------------------------
;zeigt statusreg. als bitfolge ueber usart an.
sregu:
	push	r16
	in		r16,SREG
	push	r16
	rcall	crlf
	rcall	lbit
	pop		r16
	out		SREG,r16
	pop		r16
	ret
;----------------------------------------------------------
;zeigt statusreg. als bitfolge und
;reg0.....reg32 in hex an
reg_d:
	push	r31			;sichert r31 auf stack
	push	r30			;sichert r30 auf stack
	push	r17			;sichert r17 auf stack
	push	r16			;sichert r16 auf stack
	push	r31
	push	r30
	push	r17
	push	r16
	rcall	crlf
	in		r16,SREG
	rcall	lbit		;und zeigt bits an
	rcall	crlf
	clr		r30			;start von r0 (Z adresse 0)
	clr		r31			;start von r0 (Z adresse 0)		
	ldi		r17,16		;r0....r15
rdp1:
	rcall	lreg		;zeigt r0...r15 an
	rcall	crlf
	pop		r16			;holt r16 von stack
	rcall	lbyte		;und zeigt r16 an
	rcall	blk
	pop		r16			;holt r17 von stack
	rcall	lbyte		;und zeigt r17 an
	rcall	blk
	subi	r30,-2
	ldi		r17,12
rdp2:
	rcall	lreg		;zeigt r18...r29 an
	pop		r16			;holt r30 von stack
	rcall	lbyte		;und zeigt r30 an
	rcall	blk
	pop		r16			;holt r31 von stack
	rcall	lbyte		;und zeigt r31 an
	rcall	blk
	pop		r16
	pop		r17
	pop		r30
	pop		r31
	ret

;*************************************************
;ab hier: diverse delays
;achtung: diese delays sind nur ok., wenn int. aus (cli)
;-------------------------------------------------
;delays clk = 10.24 MHz

del5us:
	push	r16
	ldi		r16,10
	rjmp	del10u_1

del9us:
	push	r16
	ldi		r16,27
	rjmp	del10u_1

del10us:
	push	r16
	ldi		r16,29
del10u_1:
	dec		r16
	brbc	1,del10u_1
	pop		r16
	ret

del13us:
	push	r16
	ldi		r16,40
	rjmp	del10u_1

del16us:
	push	r16
	ldi		r16,50
	rjmp	del10u_1

del26us:
	push	r16
	ldi		r16,81
	rjmp	del10u_1

del52us:
	push	r16
	ldi		r16,173
	rjmp	del10u_1

delr16:
	push	r16
	rjmp	del100u_1

del100us:
	push	r16
	ldi		r16,10
del100u_1:
	push	r16
	rcall	del9us
	pop		r16
	dec		r16
	brbc	1,del100u_1
	pop		r16
	ret

del470us:
	push	r16
	ldi		r16,47
	rjmp	del100u_1

del1ms:
	push	r16
	ldi		r16,100
	rjmp	del100u_1

del2ms:
	push	r16
	ldi		r16,200
	rjmp	del100u_1

del5ms:
	push	r16
	ldi		r16,5
	rjmp	del10m_1

del10ms:
	push	r16
	ldi		r16,10
del10m_1:
	push	r16
	rcall	del1ms
	pop		r16
	dec		r16
	brbc	1,del10m_1
	pop		r16
	ret
del20ms:
	push	r16
	ldi		r16,20
	rjmp	del10m_1
del40ms:
	push	r16
	rcall	del470us
	ldi		r16,40
	rjmp	del10m_1
del50ms:
	push	r16
	ldi		r16,50
	rjmp	del10m_1

del100ms:
	push	r16
	ldi		r16,101
	rjmp	del10m_1
del200ms:
	push	r16
	ldi		r16,199
	rjmp	del10m_1
del500ms:
	push	r16
	ldi		r16,5
	rjmp	del1s_1
del1s:
	push	r16
	ldi		r16,10
del1s_1:
	push	r16
	rcall	del100ms
	pop		r16
	dec		r16
	brbc	1,del1s_1
	pop		r16
	ret
del2s:
	push	r16
	ldi		r16,20
	rjmp	del1s_1
del5s:
	push	r16
	ldi		r16,50
	rjmp	del1s_1
	
;------------------------------------------------


;#######################################################
;-------------------------------------------------------
;-------------------------------------------------------
;diverse LCD-Routinen
; pins used for control (bit-banged SPI)
; change to suit your needs
#define	LCD_CS		PORTC,5		; chip-select
#define	LCD_CS_DIR	DDRC,5		; chip-select pin direction

#define	LCD_RST		PORTC,4		; reset pin
#define	LCD_RST_DIR	DDRC,4		; reset pin direction

#define	LCD_A0		PORTC,3		; register/screen select
#define	LCD_A0_DIR	DDRC,3		; register/screen select pin direction

#define	LCD_SCLK	PORTC,1		; serial clock
#define	LCD_SCLK_DIR	DDRC,1		; serial clock pin direction

#define	LCD_SID		PORTC,0		; serial data (m-s-bit 1st)
#define	LCD_SID_DIR	DDRC,0		; serial data pin direction


; commands etc for screen setup and driving
#define CMD_DISPLAY_OFF   0xAE
#define CMD_DISPLAY_ON    0xAF

#define CMD_SET_DISP_START_LINE  0x40
#define CMD_SET_PAGE  0xB0

#define CMD_SET_COLUMN_UPPER  0x10
#define CMD_SET_COLUMN_LOWER  0x01

#define CMD_SET_ADC_NORMAL  0xA0
#define CMD_SET_ADC_REVERSE 0xA1

#define CMD_SET_DISP_NORMAL 0xA6
#define CMD_SET_DISP_REVERSE 0xA7

#define CMD_SET_ALLPTS_NORMAL 0xA4
#define CMD_SET_ALLPTS_ON  0xA5
#define CMD_SET_BIAS_9 0xA2 
#define CMD_SET_BIAS_7 0xA3

#define CMD_RMW  0xE0
#define CMD_RMW_CLEAR 0xEE
#define CMD_INTERNAL_RESET  0xE2
#define CMD_SET_COM_NORMAL  0xC0
#define CMD_SET_COM_REVERSE  0xC8
#define CMD_SET_POWER_CONTROL  0x2f
#define CMD_SET_RESISTOR_RATIO  0x20
#define CMD_SET_VOLUME_FIRST  0x81
#define CMD_SET_VOLUME_SECOND  0
#define CMD_SET_STATIC_OFF  0xAC
#define CMD_SET_STATIC_ON  0xAD
#define CMD_SET_STATIC_REG  0x0
#define CMD_SET_BOOSTER_FIRST  0xF8
#define CMD_SET_BOOSTER_234  0
#define CMD_SET_BOOSTER_5  1
#define CMD_SET_BOOSTER_6  3
#define CMD_NOP  0xE3
#define CMD_TEST  0xF0




; call this on power-up to initialise the display
init_st7565:
	sbi		LCD_CS_DIR	; output
	sbi		LCD_CS		; select pin high (not selected)

	sbi		LCD_A0_DIR	; outputs
	sbi		LCD_A0		; set high

	sbi		LCD_RST_DIR	; output
	sbi		LCD_RST		; reset pin high

	sbi		LCD_SID_DIR	; output
	sbi		LCD_SID		; serial pin high

	sbi		LCD_SCLK_DIR	; output
	cbi		LCD_SCLK	; clk pin low

	cbi		LCD_CS		; LCD now selected/active
; control pins now in startup state, ready to do a hard reset on display
	cbi		LCD_RST		; pull low to reset
	rcall	del10ms		; wait 10ms
	sbi		LCD_RST		; and back high again
	sbi		LCD_CS		; and de-select
	rcall	del2ms		; 2ms

; now start the initialisation of internal registers
	ldi		temp1,CMD_SET_BIAS_9
	rcall	LCD_command

	ldi		temp1,CMD_SET_ADC_NORMAL
	rcall	LCD_command

	ldi		temp1,CMD_SET_COM_REVERSE
	rcall	LCD_command

	ldi		temp1,CMD_SET_DISP_START_LINE | 0x20
	rcall	LCD_command



	ldi		temp1,CMD_SET_POWER_CONTROL
	rcall	LCD_command
	rcall	del50ms

	ldi		temp1,(CMD_SET_RESISTOR_RATIO | 0x4)
	rcall	LCD_command
	ldi		temp1,0xf8			;booster,regulator,follower on
	rcall	LCD_command	
	ldi		temp1,0x00			;int. booster x4
	rcall	LCD_command

; next two are sent as a pair, 1st one enters a mode, 2nd is the value
	ldi		temp1,CMD_SET_VOLUME_FIRST
	rcall	LCD_command
	ldi		temp1,40		; 0..63 for contrast
	rcall	LCD_command

	rcall	LCD_clearscreen		; clears and sets to page-0, display coll-0
					; (and row/coll=0)

	ldi		temp1,CMD_DISPLAY_ON	; display ON
	rcall	LCD_command

	ret

;----------------------------------------------------
; to send comands to the display.
LCD_command:
	cbi		LCD_A0		; register select
	rjmp	reg

;sendet zeichen an das display
LCD_send:
	sbrc	flag,flg_invert	; invert
	com		temp1

;serielle (spi) ausgabe -> zeichen -> display
reg:	
	push	temp1
	push	loopcounter
	ldi		loopcounter,8	;8 bit/zeichen
	cbi		LCD_CS
LClp:
	cbi		LCD_SCLK
	rol		temp1
	brcs	LC1
	cbi		LCD_SID		; lscht op bit
	rjmp	doclk
LC1:
	sbi		LCD_SID		; setzt op bit
doclk:
	nop
	sbi		LCD_SCLK
	dec		loopcounter
	brne	LClp
	sbi		LCD_CS		; deselect display
	pop		loopcounter
	sbi		LCD_A0
	pop		temp1
	ret





; wipes the screen and sets start position to 0,0 (top left) for chars
LCD_clearscreen:
	ldi		temp1,CMD_SET_PAGE		; sets to page-0
	rcall	LCD_command
	ldi		temp1,0				; fill value (all bits off)
	rcall	fillpage			; write it to a complete page (text line)
	ldi		temp1,(CMD_SET_PAGE +1)		; page 1
	rcall	LCD_command
	ldi		temp1,0
	rcall	fillpage
	ldi		temp1,(CMD_SET_PAGE + 2)	; page 2
	rcall	LCD_command
	ldi		temp1,0
	rcall	fillpage
	ldi		temp1,(CMD_SET_PAGE + 3)
	rcall	LCD_command
	ldi		temp1,0
	rcall	fillpage
	ldi		temp1,(CMD_SET_PAGE + 4)
	rcall	LCD_command
	ldi		temp1,0
	rcall	fillpage
	ldi		temp1,(CMD_SET_PAGE + 5)
	rcall	LCD_command
	ldi		temp1,0
	rcall	fillpage
	ldi		temp1,(CMD_SET_PAGE + 6)
	rcall	LCD_command
	ldi		temp1,0
	rcall	fillpage
	ldi		temp1,(CMD_SET_PAGE + 7)
	rcall	LCD_command
	ldi		temp1,0
	rcall	fillpage
; set to top-LHS of screen
;	lcd_coll 0
	clr		temp1
	sts		Lcoll,temp1
	rcall	lcd_set_coll
;	lcd_page 0
	clr		temp1
	sts		Lpage,temp1
	rcall	lcd_set_page
	clr		temp1
	sts		row,temp1
	sts		coll,temp1
	ret


; fills a page (row) with the byte value (vertical 8-pixel strip) in temp1
fillpage:
	push	temp1
	ldi		temp1,CMD_SET_COLUMN_UPPER
	rcall	LCD_command
	ldi		temp1,CMD_SET_COLUMN_LOWER
	rcall	LCD_command
	pop		temp1

	ldi		loopcounter,129
lloop:
	rcall	LCD_send
	dec		loopcounter
	brne	lloop
	ret



;--------------------------------------------------
; sets the collum pixel location (0...127)
; addr in Lcol
; use the macro lcd_col. ie 'lcd_col 83' to set the 
; horizontal position to 83 pixels along the current page (row)
; only useful for exact horizontal character position writes.
; 'lcd_putchar' fits chars onto an exact regular grid based on
; the row/col co-ordinate which is character based.
; by using lcd_set_col, this specifies an exact horizontal position
; to write a char using 'put_LCDchar' which writes a character at the
; current horizontal location (0...127) on the current page (row) 0...7
lcd_set_coll:
	push	temp1
	lds		temp1,Lcoll
	push	temp1
	andi	temp1,0b11110000
	swap	temp1
	ori		temp1,CMD_SET_COLUMN_UPPER
	rcall	LCD_command
	pop		temp1
	andi	temp1,0b00001111
	ori		temp1,CMD_SET_COLUMN_LOWER
	rcall	LCD_command
	pop		temp1
	ret

; sets the page (row). A page is 128 vertical sections, 8-pixels high each
; use the macro lcd_page
lcd_set_page:
	lds		temp1,Lpage
lsp10:
	ori		temp1,CMD_SET_PAGE
	rcall	LCD_command
	ret

; **********************************************************************************************
; taken from AVR freaks. random num gen
;
;peret - Oct 12, 2007 - 02:12 AM
;Post subject: Here's a good, quick RNG. 
;The period (2^31-1) is prime so it's not easy to short-circuit it. 
;Half the time it takes 6 cycles and the other half it takes 15, 
;for an average of 10.5, if it's register-based as shown. 
;You can make a quicker, shorter one by using a 15 or 16 bit algorithm instead of 31. 
;How to choose the polynomial? It's a black art, but CRC polynomials generally 
;work well. 
;In fact this kind of RNG is mathematically the same as taking the CRC of an 
;infinite string of 1's. If you already have a CRC routine in your code you get an RNG for free.
;
;Code:
; 31-bit random number
; primitive polynomial order 31
; (1)0100000 10101010 10010101 10110101 (20 AA 95 B5)
; shift before XOR
rand_31:
	lsl	r4		; shift first
	rol	r5
	rol	r6
	rol	r7
	sbrs	r7,7		; test MSB	
	ret			; clear, no XOR

	push	temp2
	ldi	temp2,0xB5	; first poly byte
	eor	r4,temp2	; XOR it in
	ldi	temp2,0x95	; second byte of poly
	eor	r5,temp2
	ldi	temp2,0xAA	; same routine
	eor	r6,temp2
	ldi	temp2,0x20
	eor	r7,temp2
	pop	temp2
	ret 
; *************************************************************************************************

; gets a number between 0..9
getone:	rcall	rand_31
	mov	temp1,r6
	andi	temp1,0b00001111
	cpi	temp1,10
	brsh	getone		; just loop till in range 0..9
	ret



